@@ -39,7 +39,18 @@ from i2.wrapper import (
     include_exclude,
     rm_params,
     partialx,
-    map_names,
+    ch_names,
     func_to_method_func,
     bind_funcs_object_attrs,
+    kwargs_trans,
+)
+
+from i2.util import (
+    Literal,
+    path_extractor,
+    get_function_body,
+    lazyprop,
+    frozendict,
+    inject_method,
+    make_sentinel,
 )
