@@ -1026,19 +1026,19 @@ def nice_kinds(func):
 
 
 @double_up_as_factory
-def map_names(func=None, **old_to_new_name):
+def ch_names(func=None, **old_to_new_name):
     """Change the argument names of a function.
 
     >>> def f(w, /, x: float, y=2, *, z: int = 3):
     ...     return f"(w:={w}) + (x:={x}) * (y:={y}) ** (z:={z}) == {w + x * y ** z}"
-    >>> wrapped_f = map_names(f, w='DoubleYou', z='Zee')
+    >>> wrapped_f = ch_names(f, w='DoubleYou', z='Zee')
     >>> wrapped_f
     <i2.Wrap f(DoubleYou, /, x: float, y=2, *, Zee: int = 3)>
     >>> wrapped_f(1, 2, y=3, Zee=4)
     '(w:=1) + (x:=2) * (y:=3) ** (z:=4) == 163'
 
     Can also be used as a factory:
-    >>> @map_names(a='alpha', g='gamma')
+    >>> @ch_names(a='alpha', g='gamma')
     ... def foo(a, b, g=1):
     ...     return a + b * g
     >>> foo(alpha=1, b=2, gamma=3)
@@ -1047,6 +1047,9 @@ def map_names(func=None, **old_to_new_name):
     return Ingress.name_map(func, **old_to_new_name).wrap(func)
 
 
+map_names = ch_names  # back-compatibility alias
+
+
 def include_exclude_ingress_factory(func, include=None, exclude=None):
     """A pattern underlying any ingress that takes a subset of parameters (possibly
     reordering them).
@@ -1211,13 +1214,11 @@ def params_used_in_funcs(funcs):
     return {name for func in funcs for name in Sig(func).names}
 
 
-def required_params(func):
-    sig = Sig(func)
-    return set(sig.names) - sig.defaults.keys()
-
-
 def required_params_used_in_funcs(funcs):
-    return {name for func in funcs for name in required_params(func)}
+    return {name for func in funcs for name in Sig(func).required_names}
+
+
+from i2.signatures import _call_forgivingly
 
 
 # TODO: Should we add some explicit/validation/strict options?
@@ -1296,13 +1297,13 @@ def kwargs_trans(
     if not _recursive:
         new_kwargs = dict()
         for key, val_func in key_and_val_func.items():
-            new_kwargs[key] = call_forgivingly(val_func, **kwargs)
+            new_kwargs[key] = _call_forgivingly(val_func, (), kwargs)
         return dict(kwargs, **new_kwargs)
     else:
         if _inplace is False:
             kwargs = kwargs.copy()
         for key, val_func in key_and_val_func.items():
-            kwargs[key] = call_forgivingly(val_func, **kwargs)
+            kwargs[key] = _call_forgivingly(val_func, (), kwargs)
         return kwargs
 
 
@@ -1420,7 +1421,7 @@ def func_to_method_func(
     you need to give it to have the effect you want (the binding of those arguments
     to attributes of the instance):
 
-    >>> from i2.wrapper import map_names
+    >>> from i2.wrapper import ch_names
     >>> def func(x, y: int, z=2, *, d='bar'):
     ...     return f"{d}: {(x + y) * z}"
     >>> from dataclasses import dataclass
@@ -1428,7 +1429,7 @@ def func_to_method_func(
     ... class Klass:
     ...     a : int = 1
     ...     c : int = 3
-    ...     method_func = func_to_method_func(map_names(func, x='a', z='c'), 'a c')
+    ...     method_func = func_to_method_func(ch_names(func, x='a', z='c'), 'a c')
     >>> instance = Klass(1, 3)
     >>> instance.method_func(2, d='hello')
     'hello: 9'
